@@ -159,13 +159,15 @@ object UnitAutomation {
             return
         }
 
-        if (unit.promotions.canBePromoted() &&
+        while (unit.promotions.canBePromoted() &&
             // Restrict Human automated units from promotions via setting
                 (UncivGame.Current.settings.automatedUnitsChoosePromotions || unit.civ.isAI())) {
             val availablePromotions = unit.promotions.getAvailablePromotions()
                 .filterNot { it.hasUnique(UniqueType.SkipPromotion) }
-            if (availablePromotions.any())
-                unit.promotions.addPromotion(availablePromotions.toList().random().name)
+            if (availablePromotions.none()) break
+            unit.promotions.addPromotion(
+                availablePromotions.filter { it.hasUnique(UniqueType.FreePromotion) }.toList().randomOrNull()?.name
+                    ?: availablePromotions.toList().random().name)
         }
 
         //This allows for military units with certain civilian abilities to behave as civilians in peace and soldiers in war
