@@ -29,7 +29,7 @@ class CityFounder {
         city.isOriginalCapital = civInfo.citiesCreated == 0
         if (city.isOriginalCapital) {
             civInfo.hasEverOwnedOriginalCapital = true
-            // if you have some culture before the 1st city is found, you may want to adopt the 1st policy
+            // if you have some culture before the 1st city is founded, you may want to adopt the 1st policy
             civInfo.policies.shouldOpenPolicyPicker = true
         }
         civInfo.citiesCreated++
@@ -43,6 +43,7 @@ class CityFounder {
         city.expansion.reset()
 
         city.tryUpdateRoadStatus()
+//         civInfo.cache.updateCitiesConnectedToCapital() // Carthage cities can connect immediately
 
         val tile = city.getCenterTile()
         for (terrainFeature in tile.terrainFeatures.filter {
@@ -52,7 +53,7 @@ class CityFounder {
         })
             tile.removeTerrainFeature(terrainFeature)
 
-        tile.changeImprovement(null)
+        tile.removeImprovement()
         tile.improvementInProgress = null
 
         val ruleset = civInfo.gameInfo.ruleset
@@ -90,6 +91,7 @@ class CityFounder {
         ))
             UniqueTriggerActivation.triggerCivwideUnique(unique, civInfo, city, triggerNotificationText = "due to founding a city")
 
+
         return city
     }
 
@@ -180,7 +182,7 @@ class CityFounder {
         // names are taken from the rest of the major nations in the rule set
         val absentMajorNations: Sequence<Nation> =
                 foundingCiv.gameInfo.ruleset.nations.values.asSequence().filter { nation ->
-                    nation.isMajorCiv() && nation !in aliveMajorNations
+                    nation.isMajorCiv && nation !in aliveMajorNations
                 }
         newCityNames =
                 absentMajorNations.flatMap { nation ->
@@ -203,11 +205,10 @@ class CityFounder {
             val building = ruleset.buildings[buildingName] ?: continue
             val uniqueBuilding = civInfo.getEquivalentBuilding(building)
             if (uniqueBuilding.isBuildable(city.cityConstructions))
-                city.cityConstructions.addBuilding(uniqueBuilding.name)
+                city.cityConstructions.addBuilding(uniqueBuilding)
         }
 
         civInfo.civConstructions.tryAddFreeBuildings()
-        city.cityConstructions.addFreeBuildings()
     }
 
 
