@@ -25,6 +25,7 @@ import com.unciv.models.stats.Stats
 import com.unciv.ui.components.extensions.filterAndLogic
 import java.text.DecimalFormat
 import kotlin.math.pow
+import kotlin.math.ulp
 
 
 /**
@@ -189,7 +190,7 @@ class MapUnit : IsPartOfGameInfoSerialization {
     }
 
     val type: UnitType
-        get() = baseUnit.getType()
+        get() = baseUnit.type
 
     fun baseUnit(): BaseUnit = baseUnit
     fun getMovementString(): String =
@@ -266,9 +267,24 @@ class MapUnit : IsPartOfGameInfoSerialization {
 
         if (movement < 1) movement = 1
 
+        // Hakkapeliitta movement boost
+        // For every double-stacked tile, check if our cohabitant can boost our speed
+        // (a test `count() > 1` is no optimization - two iterations of a sequence instead of one)
+        for (boostingUnit in currentTile.getUnits()) {
+            if (boostingUnit == this) continue
+            if (boostingUnit.getMatchingUniques(UniqueType.TransferMovement)
+                    .none { matchesFilter(it.params[0]) } ) continue
+            movement = movement.coerceAtLeast(boostingUnit.getMaxMovement())
+        }
+
         return movement
     }
 
+    fun hasUnitMovedThisTurn(): Boolean {
+        val max = getMaxMovement().toFloat()
+        return currentMovement < max - max.ulp
+    }
+
     /**
      * Determines this (land or sea) unit's current maximum vision range from unit properties, civ uniques and terrain.
      * @return Maximum distance of tiles this unit may possibly see
@@ -278,15 +294,9 @@ class MapUnit : IsPartOfGameInfoSerialization {
 
         val conditionalState = StateForConditionals(civInfo = civ, unit = this)
 
-        if (isEmbarked() && !hasUnique(UniqueType.NormalVisionWhenEmbarked, conditionalState, checkCivInfoUniques = true)) {
-            return 1
-        }
-
-        visibilityRange += getMatchingUniques(UniqueType.Sight, conditionalState, checkCivInfoUniques = true)
-            .sumOf { it.params[0].toInt() }
-
-        visibilityRange += getTile().getMatchingUniques(UniqueType.Sight, conditionalState)
-            .sumOf { it.params[0].toInt() }
+        val relevantUniques = getMatchingUniques(UniqueType.Sight, conditionalState, checkCivInfoUniques = true) +
+                getTile().getMatchingUniques(UniqueType.Sight, conditionalState)
+        visibilityRange += relevantUniques.sumOf { it.params[0].toInt() }
 
         if (visibilityRange < 1) visibilityRange = 1
 
@@ -307,7 +317,10 @@ class MapUnit : IsPartOfGameInfoSerialization {
         }
 
         // Set equality automatically determines if anything changed - https://kotlinlang.org/api/latest/jvm/stdlib/kotlin.collections/-abstract-set/equals.html
-        if (updateCivViewableTiles && oldViewableTiles != viewableTiles)
+        if (updateCivViewableTiles && oldViewableTiles != viewableTiles
+                // Don't bother updating if all previous and current viewable tiles are within our borders
+                && (oldViewableTiles.any { it !in civ.cache.ourTilesAndNeighboringTiles }
+                        || viewableTiles.any { it !in civ.cache.ourTilesAndNeighboringTiles }))
             civ.cache.updateViewableTiles(explorerPosition)
     }
 
@@ -374,13 +387,14 @@ class MapUnit : IsPartOfGameInfoSerialization {
 
     fun isEmbarked(): Boolean {
         if (!baseUnit.isLandUnit()) return false
+        if (cache.canMoveOnWater) return false
         return currentTile.isWater
     }
 
     fun isInvisible(to: Civilization): Boolean {
-        if (hasUnique(UniqueType.Invisible))
+        if (hasUnique(UniqueType.Invisible) && !to.isSpectator())
             return true
-        if (hasUnique(UniqueType.InvisibleToNonAdjacent))
+        if (hasUnique(UniqueType.InvisibleToNonAdjacent) && !to.isSpectator())
             return getTile().getTilesInDistance(1).none {
                 it.getUnits().any { unit -> unit.owner == to.civName }
             }
@@ -428,6 +442,14 @@ class MapUnit : IsPartOfGameInfoSerialization {
     fun isGreatPerson() = baseUnit.isGreatPerson()
     fun isGreatPersonOfType(type: String) = baseUnit.isGreatPersonOfType(type)
 
+    fun getDistanceToEnemyUnit(maxDist: Int): Int? {
+        for (i in 1..maxDist) {
+            if (currentTile.getTilesAtDistance(i).any {it.militaryUnit != null
+                && it.militaryUnit!!.civ.isAtWarWith(civ) })
+                return i
+        }
+        return null
+    }
     //endregion
 
     //region state-changing functions
@@ -504,6 +526,7 @@ class MapUnit : IsPartOfGameInfoSerialization {
         var healing = when {
             tile.isCityCenter() -> 25
             tile.isWater && isFriendlyTerritory && (baseUnit.isWaterUnit() || isTransported) -> 20 // Water unit on friendly water
+            tile.isWater && isFriendlyTerritory && cache.canMoveOnWater -> 20 // Treated as a water unit on friendly water
             tile.isWater -> 0 // All other water cases
             isFriendlyTerritory -> 20 // Allied territory
             tile.getOwner() == null -> 10 // Neutral territory
@@ -553,10 +576,9 @@ class MapUnit : IsPartOfGameInfoSerialization {
     fun gift(recipient: Civilization) {
         civ.units.removeUnit(this)
         civ.cache.updateViewableTiles()
-        // all transported units should be destroyed as well
+        // all transported units should be gift as well
         currentTile.getUnits().filter { it.isTransported && isTransportTypeOf(it) }
-            .toList() // because we're changing the list
-            .forEach { unit -> unit.destroy() }
+            .forEach { unit -> unit.gift(recipient) }
         assignOwner(recipient)
         recipient.cache.updateViewableTiles()
     }
@@ -643,7 +665,7 @@ class MapUnit : IsPartOfGameInfoSerialization {
     }
 
     private fun clearEncampment(tile: Tile) {
-        tile.changeImprovement(null)
+        tile.removeImprovement()
 
         // Notify City-States that this unit cleared a Barbarian Encampment, required for quests
         civ.gameInfo.getAliveCityStates()
@@ -694,7 +716,7 @@ class MapUnit : IsPartOfGameInfoSerialization {
     }
 
     private fun getAncientRuinBonus(tile: Tile) {
-        tile.changeImprovement(null)
+        tile.removeImprovement()
         civ.ruinsManager.selectNextRuinsReward(this)
     }
 
@@ -784,6 +806,10 @@ class MapUnit : IsPartOfGameInfoSerialization {
         return true
     }
 
+    /** Gets a Nuke's blast radius from the BlastRadius unique, defaulting to 2. No check whether the unit actually is a Nuke. */
+    fun getNukeBlastRadius() = getMatchingUniques(UniqueType.BlastRadius)
+        // Don't check conditionals as these are not supported
+        .firstOrNull()?.params?.get(0)?.toInt() ?: 2
 
     private fun isAlly(otherCiv: Civilization): Boolean {
         return otherCiv == civ
@@ -796,15 +822,14 @@ class MapUnit : IsPartOfGameInfoSerialization {
         return filter.filterAndLogic { matchesFilter(it) } // multiple types at once - AND logic. Looks like:"{Military} {Land}"
             ?: when (filter) {
 
-            // todo: unit filters should be adjectives, fitting "[filterType] units"
-            // This means converting "wounded units" to "Wounded", "Barbarians" to "Barbarian"
-            "Wounded", "wounded units" -> health < 100
+            Constants.wounded, "wounded units" -> health < 100
             Constants.barbarians, "Barbarian" -> civ.isBarbarian()
             "City-State" -> civ.isCityState()
-            "Embarked" -> isEmbarked()
+            Constants.embarked -> isEmbarked()
             "Non-City" -> true
             else -> {
                 if (baseUnit.matchesFilter(filter)) return true
+                if (civ.nation.matchesFilter(filter)) return true
                 if (tempUniquesMap.containsKey(filter)) return true
                 return false
             }
@@ -814,7 +839,7 @@ class MapUnit : IsPartOfGameInfoSerialization {
     fun canBuildImprovement(improvement: TileImprovement, tile: Tile = currentTile): Boolean {
         // Workers (and similar) should never be able to (instantly) construct things, only build them
         // HOWEVER, they should be able to repair such things if they are pillaged
-        if (improvement.turnsToBuild == 0
+        if (improvement.turnsToBuild == -1
             && improvement.name != Constants.cancelImprovementOrder
             && tile.improvementInProgress != improvement.name
         ) return false
