@@ -1,5 +1,6 @@
 package com.unciv.logic.map.mapunit
 
+import com.unciv.logic.UncivShowableException
 import com.unciv.models.ruleset.RejectionReasonType
 import com.unciv.models.ruleset.unique.StateForConditionals
 import com.unciv.models.ruleset.unique.UniqueType
@@ -11,11 +12,13 @@ class UnitUpgradeManager(val unit:MapUnit) {
 
     /** Returns FULL upgrade path, without checking what we can or cannot build currently.
      * Does not contain current baseunit, so will be empty if no upgrades. */
-    private fun getUpgradePath(): List<BaseUnit>{
+    private fun getUpgradePath(): Iterable<BaseUnit> {
         var currentUnit = unit.baseUnit
-        val upgradeList = arrayListOf<BaseUnit>()
-        while (currentUnit.upgradesTo != null){
+        val upgradeList = linkedSetOf<BaseUnit>()
+        while (currentUnit.upgradesTo != null) {
             val nextUpgrade = unit.civ.getEquivalentUnit(currentUnit.upgradesTo!!)
+            if (nextUpgrade in upgradeList)
+                throw(UncivShowableException("Circular or self-referencing upgrade path for ${currentUnit.name}"))
             currentUnit = nextUpgrade
             upgradeList.add(currentUnit)
         }
@@ -32,7 +35,7 @@ class UnitUpgradeManager(val unit:MapUnit) {
         fun isInvalidUpgradeDestination(baseUnit: BaseUnit): Boolean{
             if (baseUnit.requiredTech != null && !unit.civ.tech.isResearched(baseUnit.requiredTech!!))
                 return true
-            if (baseUnit.getMatchingUniques(UniqueType.OnlyAvailableWhen).any {
+            if (baseUnit.getMatchingUniques(UniqueType.OnlyAvailableWhen, StateForConditionals.IgnoreConditionals).any {
                         !it.conditionalsApply(StateForConditionals(unit.civ, unit = unit ))
                     }) return true
             return false
