@@ -19,8 +19,8 @@ class GameParameters : IsPartOfGameInfoSerialization { // Default values are the
     var minNumberOfPlayers = 3
     var maxNumberOfPlayers = 3
     var players = ArrayList<Player>().apply {
-        add(Player().apply { playerType = PlayerType.Human })
-        for (i in 1..3) add(Player())
+        add(Player(playerType = PlayerType.Human))
+        repeat(3) { add(Player()) }
     }
     var randomNumberOfCityStates = false
     var minNumberOfCityStates = 6
@@ -43,12 +43,15 @@ class GameParameters : IsPartOfGameInfoSerialization { // Default values are the
     var startingEra = "Ancient era"
 
     var isOnlineMultiplayer = false
+    var multiplayerServerUrl: String? = null
     var anyoneCanSpectate = true
     var baseRuleset: String = BaseRuleset.Civ_V_GnK.fullName
     var mods = LinkedHashSet<String>()
 
     var maxTurns = 500
 
+    var acceptedModCheckErrors = ""
+
     fun clone(): GameParameters {
         val parameters = GameParameters()
         parameters.difficulty = difficulty
@@ -74,10 +77,12 @@ class GameParameters : IsPartOfGameInfoSerialization { // Default values are the
         parameters.victoryTypes = ArrayList(victoryTypes)
         parameters.startingEra = startingEra
         parameters.isOnlineMultiplayer = isOnlineMultiplayer
+        parameters.multiplayerServerUrl = multiplayerServerUrl
         parameters.anyoneCanSpectate = anyoneCanSpectate
         parameters.baseRuleset = baseRuleset
         parameters.mods = LinkedHashSet(mods)
         parameters.maxTurns = maxTurns
+        parameters.acceptedModCheckErrors = acceptedModCheckErrors
         return parameters
     }
 
@@ -86,11 +91,9 @@ class GameParameters : IsPartOfGameInfoSerialization { // Default values are the
             yield("$difficulty $speed $startingEra")
             yield("${players.count { it.playerType == PlayerType.Human }} ${PlayerType.Human}")
             yield("${players.count { it.playerType == PlayerType.AI }} ${PlayerType.AI}")
-            yield("$minNumberOfCityStates Min CS")
-            yield("$maxNumberOfCityStates Max CS")
-            yield("$numberOfCityStates CS")
-            if (randomNumberOfPlayers) yield("Random number of Players")
-            if (randomNumberOfCityStates) yield("Random number of City-States")
+            if (randomNumberOfPlayers) yield("Random number of Players: $minNumberOfPlayers..$maxNumberOfPlayers")
+            if (randomNumberOfCityStates) yield("Random number of City-States: $minNumberOfCityStates..$maxNumberOfCityStates")
+            else yield("$numberOfCityStates CS")
             if (isOnlineMultiplayer) yield("Online Multiplayer")
             if (noBarbarians) yield("No barbs")
             if (ragingBarbarians) yield("Raging barbs")
