@@ -11,9 +11,10 @@ import com.unciv.models.metadata.BaseRuleset
 import com.unciv.models.ruleset.Ruleset
 import com.unciv.models.ruleset.RulesetCache
 import com.unciv.models.ruleset.unique.Unique
+import com.unciv.ui.components.ColorMarkupLabel
+import com.unciv.ui.components.extensions.toLabel
 import com.unciv.ui.images.ImageGetter
 import com.unciv.ui.screens.basescreen.BaseScreen
-import com.unciv.ui.components.extensions.toLabel
 import com.unciv.utils.Log
 import kotlin.math.max
 
@@ -35,7 +36,7 @@ import kotlin.math.max
  *  Special cases:
  *  - Standalone [image][extraImage] from atlas or from ExtraImages
  *  - A separator line ([separator])
- *  - Automatic external links (no [text] but [link] begins with a URL protocol)
+ *  - Automatic external links ([link] begins with a URL protocol)
  */
 class FormattedLine (
     /** Text to display. */
@@ -59,7 +60,7 @@ class FormattedLine (
     val indent: Int = 0,
     /** Defines vertical padding between rows, defaults to 5f. */
     val padding: Float = Float.NaN,
-    /** Sets text color, accepts Java names or 6/3-digit web colors (e.g. #FFA040). */
+    /** Sets text color, accepts 6/3-digit web colors (e.g. #FFA040). */
     val color: String = "",
     /** Renders a separator line instead of text. Can be combined only with [color] and [size] (line width, default 2) */
     val separator: Boolean = false,
@@ -258,7 +259,7 @@ class FormattedLine (
             size == Int.MIN_VALUE -> Constants.defaultFontSize
             else -> size
         }
-        val labelColor = if(starred) defaultColor else displayColor
+        val labelColor = if (starred) defaultColor else displayColor
 
         val table = Table(BaseScreen.skin)
         var iconCount = 0
@@ -285,8 +286,10 @@ class FormattedLine (
                 else -> (indent-1) * indentPad +
                         indentOneAtNumIcons * (minIconSize + iconPad) + iconPad - usedWidth
             }
-            val label = if (fontSize == Constants.defaultFontSize && labelColor == defaultColor) textToDisplay.toLabel()
-            else textToDisplay.toLabel(labelColor,fontSize)
+            val label = if ('«' in textToDisplay)
+                ColorMarkupLabel(textToDisplay, fontSize, hideIcons = iconCount != 0)
+            else
+                textToDisplay.toLabel(labelColor, fontSize, hideIcons = iconCount != 0)
             label.wrap = !centered && labelWidth > 0f
             label.setAlignment(align)
             if (labelWidth == 0f)
