@@ -20,8 +20,8 @@ import com.unciv.ui.audio.MusicMood
 import com.unciv.ui.audio.MusicTrackChooserFlags
 import com.unciv.ui.components.TabbedPager
 import com.unciv.ui.components.extensions.center
-import com.unciv.ui.components.extensions.onClick
 import com.unciv.ui.components.extensions.toLabel
+import com.unciv.ui.components.input.onClick
 import com.unciv.ui.components.tilegroups.TileGroup
 import com.unciv.ui.components.tilegroups.TileSetStrings
 import com.unciv.ui.images.ImageGetter
@@ -208,7 +208,7 @@ class MapEditorEditImprovementsTab(
         val eraser = FormattedLine("Remove improvement", icon = eraserIcon, size = 32, iconCrossed = true)
         add(eraser.render(0f).apply { onClick {
             editTab.setBrush("Remove improvement", eraserIcon, true) { tile ->
-                tile.changeImprovement(null)
+                tile.removeImprovement()
                 tile.removeRoad()
             }
         } }).padBottom(0f).row()
@@ -281,18 +281,24 @@ class MapEditorEditStartsTab(
         val eraserIcon = "Nation/${firstNation.name}"
         val eraser = FormattedLine("Remove starting locations", icon = eraserIcon, size = 24, iconCrossed = true)
         add(eraser.render(0f).apply { onClick {
-            editTab.setBrush(BrushHandlerType.Direct, "Remove starting locations", eraserIcon, true) { tile ->
+            editTab.setBrush(BrushHandlerType.Direct, "Remove", eraserIcon, true) { tile ->
                 tile.tileMap.removeStartingLocations(tile.position)
             }
         } }).padBottom(0f).row()
-
+        // Create the nation list with the spectator nation included
+        // We use Nation/Spectator because it hasn't been used yet and we need an icon within the Nation.
+        val anyCiv = FormattedLine("Any Civ starting location", Constants.spectator, "Nation/Spectator", size = 24)
+        val nations = getNations().toList()
+        val nationsToAdd = ArrayList<FormattedLine>(1 + nations.count())
+        nationsToAdd.add((anyCiv)) // An Civ starting location should be first
+        nationsToAdd.addAll(nations)
         add(
             MarkupRenderer.render(
-            getNations(),
+            nationsToAdd,
             iconDisplay = FormattedLine.IconDisplay.NoLink
         ) {
             UncivGame.Current.musicController.chooseTrack(it, MusicMood.Theme, MusicTrackChooserFlags.setSpecific)
-            editTab.setBrush(BrushHandlerType.Direct, it, "Nation/$it") { tile ->
+            editTab.setBrush(BrushHandlerType.Direct, if (it ==Constants.spectator)  "Any Civ" else it, "Nation/$it") { tile ->
                 // toggle the starting location here, note this allows
                 // both multiple locations per nation and multiple nations per tile
                 if (!tile.tileMap.addStartingLocation(it, tile))
@@ -304,7 +310,7 @@ class MapEditorEditStartsTab(
     private fun allowedNations() = ruleset.nations.values.asSequence()
         .filter { it.name !in disallowNations && !it.hasUnique(UniqueType.CityStateDeprecated) }
     private fun getNations() = allowedNations()
-        .sortedWith(compareBy<Nation>{ it.isCityState }.thenBy(collator) { it.name.tr() })
+        .sortedWith(compareBy<Nation>{ it.isCityState }.thenBy(collator) { it.name.tr(hideIcons = true) })
         .map { FormattedLine("[${it.name}] starting location", it.name, "Nation/${it.name}", size = 24) }
         .asIterable()
 
