@@ -22,10 +22,10 @@ import com.unciv.ui.components.UncivSlider
 import com.unciv.ui.components.WrappableLabel
 import com.unciv.ui.components.extensions.addSeparator
 import com.unciv.ui.components.extensions.darken
-import com.unciv.ui.components.input.onClick
 import com.unciv.ui.components.extensions.pad
 import com.unciv.ui.components.extensions.toLabel
 import com.unciv.ui.components.extensions.toTextButton
+import com.unciv.ui.components.input.onClick
 import com.unciv.ui.popups.ToastPopup
 import com.unciv.ui.screens.basescreen.BaseScreen
 import com.unciv.ui.screens.civilopediascreen.CivilopediaScreen
@@ -95,8 +95,9 @@ class MapEditorViewTab(
 
         val area = tileMap.values.size
         val waterPercent = (tileMap.values.count { it.isWater } * 100f / area).toInt()
+        val impassablePercent = (tileMap.values.count { it.isImpassible() } * 100f / area).toInt()
         val continents = tileMap.continentSizes.size
-        val statsText = "Area: [$area] tiles, [$waterPercent]% water, [$continents] continents/islands"
+        val statsText = "Area: [$area] tiles, [$waterPercent]% water, [$impassablePercent]% impassable, [$continents] continents/islands"
         val statsLabel = WrappableLabel(statsText, labelWidth)
         add(statsLabel.apply { wrap = true }).row()
 
