@@ -15,19 +15,21 @@ import com.unciv.logic.civilization.diplomacy.DiplomaticStatus
 import com.unciv.logic.civilization.diplomacy.RelationshipLevel
 import com.unciv.logic.map.HexMath
 import com.unciv.models.ruleset.Policy.PolicyBranchType
-import com.unciv.ui.images.ImageGetter
-import com.unciv.ui.screens.diplomacyscreen.DiplomacyScreen
+import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.ui.components.AutoScrollPane
-import com.unciv.ui.screens.basescreen.BaseScreen
+import com.unciv.ui.components.ColorMarkupLabel
 import com.unciv.ui.components.Fonts
 import com.unciv.ui.components.UncivTooltip.Companion.addTooltip
 import com.unciv.ui.components.extensions.addBorder
 import com.unciv.ui.components.extensions.addSeparator
 import com.unciv.ui.components.extensions.addSeparatorVertical
 import com.unciv.ui.components.extensions.center
-import com.unciv.ui.components.extensions.onClick
 import com.unciv.ui.components.extensions.toLabel
 import com.unciv.ui.components.extensions.toTextButton
+import com.unciv.ui.components.input.onClick
+import com.unciv.ui.images.ImageGetter
+import com.unciv.ui.screens.basescreen.BaseScreen
+import com.unciv.ui.screens.diplomacyscreen.DiplomacyScreen
 import kotlin.math.roundToInt
 
 class GlobalPoliticsOverviewTable (
@@ -37,9 +39,10 @@ class GlobalPoliticsOverviewTable (
 ) : EmpireOverviewTab(viewingPlayer, overviewScreen) {
 
     class DiplomacyTabPersistableData(
+        var showDiagram: Boolean = false,
         var includeCityStates: Boolean = false
     ) : EmpireOverviewTabPersistableData() {
-        override fun isEmpty() = !includeCityStates
+        override fun isEmpty() = !showDiagram && !includeCityStates
     }
     override val persistableData = (persistedData as? DiplomacyTabPersistableData) ?: DiplomacyTabPersistableData()
 
@@ -57,18 +60,20 @@ class GlobalPoliticsOverviewTable (
     private var undefeatedCivs = sequenceOf<Civilization>()
     private var defeatedCivs = sequenceOf<Civilization>()
 
-    private var relevantCivsCount = 0  // includes unknown civs
+    private var relevantCivsCount = "?"  // includes unknown civs if player allowed to know
     private var showDiplomacyGroup = false
     private var portraitMode = false
 
 
     init {
-        updatePoliticsTable()
+        if (persistableData.showDiagram) updateDiagram()
+        else updatePoliticsTable()
     }
 
     private fun updatePoliticsTable() {
+        persistableData.showDiagram = false
         clear()
-        getFixedContent().clear()
+        fixedContent.clear()
 
         val diagramButton = "Show diagram".toTextButton().onClick(::updateDiagram)
 
@@ -84,18 +89,14 @@ class GlobalPoliticsOverviewTable (
             add("Relations".toLabel()).row()
             add(diagramButton).pad(10f)
         })
-        row()
-        addSeparator(Color.GRAY)
 
         createGlobalPoliticsTable()
     }
 
     private fun createGlobalPoliticsTable() {
-        val civilizations = mutableListOf<Civilization>()
-        civilizations.add(viewingPlayer)
-        civilizations.addAll(viewingPlayer.getKnownCivs())
-        civilizations.removeAll(civilizations.filter { it.isBarbarian() || it.isCityState() || it.isSpectator() })
-        for (civ in civilizations) {
+        for (civ in sequenceOf(viewingPlayer) + viewingPlayer.diplomacyFunctions.getKnownCivsSorted(includeCityStates = false)) {
+            addSeparator(Color.GRAY)
+
             // civ image
             add(ImageGetter.getNationPortrait(civ.nation, 100f)).pad(20f)
 
@@ -118,9 +119,6 @@ class GlobalPoliticsOverviewTable (
 
             //politics
             add(getPoliticsOfCivTable(civ)).pad(20f)
-
-            if (civilizations.indexOf(civ) != civilizations.lastIndex)
-                addSeparator(Color.GRAY)
         }
     }
 
@@ -128,7 +126,7 @@ class GlobalPoliticsOverviewTable (
         val civInfoTable = Table(skin)
         val leaderName = civ.nation.leaderName
         civInfoTable.add(leaderName.toLabel(fontSize = 30)).row()
-        civInfoTable.add(civ.civName.toLabel()).row()
+        civInfoTable.add(civ.civName.toLabel(hideIcons = true)).row()
         civInfoTable.add(civ.tech.era.name.toLabel()).row()
         return civInfoTable
     }
@@ -160,7 +158,7 @@ class GlobalPoliticsOverviewTable (
                         worldScreen.mapHolder.setCenterPosition(wonder.location.position)
                     }
                 }
-                wonderTable.add(wonderName).row()
+                wonderTable.add(wonderName).left().row()
             }
         }
 
@@ -183,18 +181,21 @@ class GlobalPoliticsOverviewTable (
         // wars
         for (otherCiv in civ.getKnownCivs()) {
             if (civ.isAtWarWith(otherCiv)) {
-                val warText = "At war with [${getCivName(otherCiv)}]".toLabel()
-                warText.color = Color.RED
+                val warText = ColorMarkupLabel("At war with [${getCivName(otherCiv)}]", Color.RED)
                 politicsTable.add(warText).row()
             }
         }
         politicsTable.row()
 
-        // declaration of friendships
+        // defensive pacts and declaration of friendships 
         for (otherCiv in civ.getKnownCivs()) {
-            if (civ.diplomacy[otherCiv.civName]?.hasFlag(DiplomacyFlags.DeclarationOfFriendship) == true) {
-                val friendText = "Friends with [${getCivName(otherCiv)}]".toLabel()
-                friendText.color = Color.GREEN
+            if (civ.diplomacy[otherCiv.civName]?.hasFlag(DiplomacyFlags.DefensivePact) == true) {
+                val friendText = ColorMarkupLabel("Defensive pact with [${getCivName(otherCiv)}]", Color.CYAN)
+                val turnsLeftText = " (${civ.diplomacy[otherCiv.civName]?.getFlag(DiplomacyFlags.DefensivePact)} ${Fonts.turn})".toLabel()
+                politicsTable.add(friendText)
+                politicsTable.add(turnsLeftText).row()
+            } else if (civ.diplomacy[otherCiv.civName]?.hasFlag(DiplomacyFlags.DeclarationOfFriendship) == true) {
+                val friendText = ColorMarkupLabel("Friends with [${getCivName(otherCiv)}]", Color.GREEN)
                 val turnsLeftText = " (${civ.diplomacy[otherCiv.civName]?.getFlag(DiplomacyFlags.DeclarationOfFriendship)} ${Fonts.turn})".toLabel()
                 politicsTable.add(friendText)
                 politicsTable.add(turnsLeftText).row()
@@ -205,8 +206,7 @@ class GlobalPoliticsOverviewTable (
         // denounced civs
         for (otherCiv in civ.getKnownCivs()) {
             if (civ.diplomacy[otherCiv.civName]?.hasFlag(DiplomacyFlags.Denunciation) == true) {
-                val denouncedText = "Denounced [${getCivName(otherCiv)}]".toLabel()
-                denouncedText.color = Color.RED
+                val denouncedText = ColorMarkupLabel("Denounced [${getCivName(otherCiv)}]", Color.RED)
                 val turnsLeftText = "(${civ.diplomacy[otherCiv.civName]?.getFlag(DiplomacyFlags.Denunciation)} ${Fonts.turn})".toLabel()
                 politicsTable.add(denouncedText)
                 politicsTable.add(turnsLeftText).row()
@@ -217,8 +217,7 @@ class GlobalPoliticsOverviewTable (
         //allied CS
         for (cityState in gameInfo.getAliveCityStates()) {
             if (cityState.diplomacy[civ.civName]?.isRelationshipLevelEQ(RelationshipLevel.Ally) == true) {
-                val alliedText = "Allied with [${getCivName(cityState)}]".toLabel()
-                alliedText.color = Color.GREEN
+                val alliedText = ColorMarkupLabel("Allied with [${getCivName(cityState)}]", Color.CYAN)
                 politicsTable.add(alliedText).row()
             }
         }
@@ -230,6 +229,7 @@ class GlobalPoliticsOverviewTable (
 
     // Refresh content and determine landscape/portrait layout
     private fun updateDiagram() {
+        persistableData.showDiagram = true
         val politicsButton = "Show global politics".toTextButton().onClick(::updatePoliticsTable)
 
         val toggleCityStatesButton: TextButton = Constants.cityStates.toTextButton().apply {
@@ -248,9 +248,12 @@ class GlobalPoliticsOverviewTable (
             add(civTableScroll.addBorder(2f, Color.WHITE)).pad(10f)
         }
 
-        relevantCivsCount = gameInfo.civilizations.count {
-            !it.isSpectator() && !it.isBarbarian() && (persistableData.includeCityStates || !it.isCityState())
-        }
+        val hideCivsCount = viewingPlayer.hideCivCount() ||
+            persistableData.includeCityStates && viewingPlayer.hideCityStateCount()
+        relevantCivsCount = if (hideCivsCount) "?"
+            else gameInfo.civilizations.count {
+                !it.isSpectator() && !it.isBarbarian() && (persistableData.includeCityStates || !it.isCityState())
+            }.toString()
         undefeatedCivs = sequenceOf(viewingPlayer) +
                 viewingPlayer.diplomacyFunctions.getKnownCivsSorted(persistableData.includeCityStates)
         defeatedCivs = viewingPlayer.diplomacyFunctions.getKnownCivsSorted(persistableData.includeCityStates, true)
@@ -292,6 +295,15 @@ class GlobalPoliticsOverviewTable (
         civTableScroll.setScrollingDisabled(portraitMode, portraitMode)
     }
 
+    /** Same as [Civilization.hideCivCount] but for City-States instead of Major Civs */
+    private fun Civilization.hideCityStateCount(): Boolean {
+        if (!gameInfo.gameParameters.randomNumberOfCityStates) return false
+        val knownCivs = 1 + getKnownCivs().count { it.isCityState() }
+        if (knownCivs >= gameInfo.gameParameters.maxNumberOfCityStates) return false
+        if (hasUnique(UniqueType.OneTimeRevealEntireMap)) return false
+        return true
+    }
+
     private fun updateCivTable(columns: Int) = civTable.apply {
         clear()
         addTitleInfo(columns)
@@ -303,7 +315,7 @@ class GlobalPoliticsOverviewTable (
     private fun getCivMiniTable(civInfo: Civilization): Table {
         val table = Table()
         table.add(ImageGetter.getNationPortrait(civInfo.nation, 25f)).pad(5f)
-        table.add(civInfo.civName.toLabel()).left().padRight(10f)
+        table.add(civInfo.civName.toLabel(hideIcons = true)).left().padRight(10f)
         table.touchable = Touchable.enabled
         table.onClick {
             if (civInfo.isDefeated() || viewingPlayer.isSpectator() || civInfo == viewingPlayer) return@onClick
@@ -328,6 +340,7 @@ class GlobalPoliticsOverviewTable (
             .pad(5f).colspan(columns).row()
         if (count == 0) return
         addSeparator()
+
         var currentColumn = 0
         var lastCivWasMajor = false
         fun advanceCols(delta: Int) {
@@ -338,6 +351,7 @@ class GlobalPoliticsOverviewTable (
             }
             lastCivWasMajor = delta == 2
         }
+
         for (civ in civs) {
             if (lastCivWasMajor && civ.isCityState())
                 advanceCols(columns)
@@ -426,6 +440,10 @@ class GlobalPoliticsOverviewTable (
                         width = 2f)
 
                     statusLine.color = if (diplomacy.diplomaticStatus == DiplomaticStatus.War) Color.RED
+                    else if (diplomacy.diplomaticStatus == DiplomaticStatus.DefensivePact
+                        || (diplomacy.civInfo.isCityState() && diplomacy.civInfo.getAllyCiv() == diplomacy.otherCivName)
+                        || (diplomacy.otherCiv().isCityState() && diplomacy.otherCiv().getAllyCiv() == diplomacy.civInfo.civName)
+                    ) Color.CYAN
                     else diplomacy.relationshipLevel().color
 
                     if (!civLines.containsKey(civ.civName)) civLines[civ.civName] = mutableSetOf()
