@@ -9,7 +9,6 @@ import com.unciv.logic.battle.Battle
 import com.unciv.logic.city.City
 import com.unciv.logic.civilization.AlertType
 import com.unciv.logic.civilization.Civilization
-import com.unciv.logic.civilization.LocationAction
 import com.unciv.logic.civilization.NotificationCategory
 import com.unciv.logic.civilization.PopupAlert
 import com.unciv.logic.civilization.diplomacy.DiplomaticModifiers
@@ -64,6 +63,10 @@ class AlertPopup(
     private fun getCity(cityId: String) = gameInfo.getCities().first { it.id == cityId }
     //endregion
 
+    // This redirects all addCloseButton uses with only text and no action to accept the space key
+    private fun addCloseButton(text: String = Constants.close) =
+        addCloseButton(text, KeyboardBinding.NextTurnAlternate, null)
+
     init {
         // This makes the buttons fill up available width. See comments in #9559.
         // To implement a middle ground, I would either simply replace growX() with minWidth(240f) or so,
@@ -126,8 +129,8 @@ class AlertPopup(
             player.getDiplomacyManager(bullyOrAttacker).sideWithCityState()
         }.row()
         addCloseButton("Very well.", KeyboardBinding.Cancel) {
-            val capitalLocation = LocationAction(cityState.cities.asSequence().map { it.location }) // in practice 0 or 1 entries, that's OK
-            player.addNotification("You have broken your Pledge to Protect [${cityState.civName}]!", capitalLocation, NotificationCategory.Diplomacy, cityState.civName)
+            player.addNotification("You have broken your Pledge to Protect [${cityState.civName}]!",
+                cityState.cityStateFunctions.getNotificationActions(), NotificationCategory.Diplomacy, cityState.civName)
             cityState.cityStateFunctions.removeProtectorCiv(player, forced = true)
         }.row()
     }
@@ -381,7 +384,7 @@ class AlertPopup(
 
     private fun addQuestionAboutTheCity(cityName: String) {
         addGoodSizedLabel("What would you like to do with the city of [$cityName]?",
-            Constants.headingFontSize).padBottom(20f).row()
+            Constants.headingFontSize, hideIcons = true).padBottom(20f).row()
     }
 
     private fun addDestroyOption(destroyAction: () -> Unit) {
