@@ -5,40 +5,68 @@ import com.badlogic.gdx.math.Interpolation
 import com.badlogic.gdx.math.Vector2
 import com.badlogic.gdx.scenes.scene2d.Actor
 import com.badlogic.gdx.scenes.scene2d.Group
+import com.badlogic.gdx.scenes.scene2d.Touchable
 import com.badlogic.gdx.scenes.scene2d.actions.Actions
 import com.badlogic.gdx.scenes.scene2d.actions.FloatAction
 import com.badlogic.gdx.scenes.scene2d.actions.RelativeTemporalAction
 import com.badlogic.gdx.scenes.scene2d.actions.RepeatAction
 import com.badlogic.gdx.scenes.scene2d.actions.SequenceAction
+import com.badlogic.gdx.scenes.scene2d.actions.TemporalAction
 import com.badlogic.gdx.scenes.scene2d.ui.Image
+import com.badlogic.gdx.scenes.scene2d.ui.Stack
 import com.badlogic.gdx.scenes.scene2d.ui.Table
+import com.badlogic.gdx.scenes.scene2d.ui.WidgetGroup
+import com.badlogic.gdx.utils.Align
 import com.unciv.UncivGame
 import com.unciv.logic.battle.ICombatant
 import com.unciv.logic.battle.MapUnitCombatant
 import com.unciv.logic.map.HexMath
+import com.unciv.ui.components.extensions.toLabel
 import com.unciv.ui.components.tilegroups.TileSetStrings
 import com.unciv.ui.images.ImageGetter
 import com.unciv.ui.screens.worldscreen.WorldScreen
 
-object BattleTableHelpers {
 
-    class FlashRedAction(start:Float, end:Float, private val actorsToOriginalColors:Map<Actor, Color>) : FloatAction(start, end, 0.2f, Interpolation.sine){
+object BattleTableHelpers {
+    /** Duration of the red-tint transition, used once per direction */
+    private const val flashRedDuration = 0.2f
+    /** Duration of the attacker displacement, used once per direction */
+    private const val moveActorsDuration = 0.3f
+    /** Max distance of the attacker displacement, in world coords */
+    private const val moveActorsDisplacement = 10f
+    /** If a mod provides attack animations (e.g. swinging a sword), they're played with this duration per frame */
+    private const val attackAnimationFrameDuration = 0.1f
+    /** Duration a damage number label is visible */
+    private const val damageLabelDuration = 1.2f
+    /** Size of a damage number label - currently fixed independednt of map zoom */
+    private const val damageLabelFontSize = 40
+    /** Total distance a damage number label is displaced upwards during that time in world coords */
+    private const val damageLabelDisplacement = 90f
+
+
+    class FlashRedAction(
+        start: Float, end: Float,
+        private val actorsToOriginalColors: Map<Actor, Color>
+    ) : FloatAction(start, end, flashRedDuration, Interpolation.sine) {
         private fun updateRedPercent(percent: Float) {
             for ((actor, color) in actorsToOriginalColors)
-                actor.color = color.cpy().lerp(Color.RED, start+percent*(end-start))
+                actor.color = color.cpy().lerp(Color.RED, start + percent * (end - start))
         }
 
         override fun update(percent: Float) = updateRedPercent(percent)
     }
 
 
-    class MoveActorsAction(private val actorsToMove:List<Actor>, private val movementVector: Vector2) : RelativeTemporalAction(){
+    class MoveActorsAction(
+        private val actorsToMove: List<Actor>,
+        private val movementVector: Vector2
+    ) : RelativeTemporalAction() {
         init {
-            duration = 0.3f
+            duration = moveActorsDuration
             interpolation = Interpolation.sine
         }
         override fun updateRelative(percentDelta: Float) {
-            for (actor in actorsToMove){
+            for (actor in actorsToMove) {
                 actor.moveBy(movementVector.x * percentDelta, movementVector.y * percentDelta)
             }
         }
@@ -46,21 +74,21 @@ object BattleTableHelpers {
 
 
     class AttackAnimationAction(
-        val attacker: ICombatant,
-        val defenderActors: List<Actor>,
-        val currentTileSetStrings: TileSetStrings
-    ): SequenceAction(){
+        private val attacker: ICombatant,
+        defenderActors: List<Actor>,
+        private val currentTileSetStrings: TileSetStrings
+    ): SequenceAction() {
         init {
             if (defenderActors.any()) {
                 val attackAnimationLocation = getAttackAnimationLocation()
-                if (attackAnimationLocation != null){
+                if (attackAnimationLocation != null) {
                     var i = 1
-                    while (ImageGetter.imageExists(attackAnimationLocation+i)){
-                        val image = ImageGetter.getImage(attackAnimationLocation+i)
+                    while (ImageGetter.imageExists(attackAnimationLocation + i)){
+                        val image = ImageGetter.getImage(attackAnimationLocation + i)
                         addAction(Actions.run {
                             defenderActors.first().parent.addActor(image)
                         })
-                        addAction(Actions.delay(0.1f))
+                        addAction(Actions.delay(attackAnimationFrameDuration))
                         addAction(Actions.removeActor(image))
                         i++
                     }
@@ -68,25 +96,55 @@ object BattleTableHelpers {
             }
         }
 
-        private fun getAttackAnimationLocation(): String?{
+        private fun getAttackAnimationLocation(): String? {
+            fun TileSetStrings.getLocation(name: String) = getString(unitsLocation, name, "-attack-")
+
             if (attacker is MapUnitCombatant) {
-                val unitSpecificAttackAnimationLocation =
-                        currentTileSetStrings.getString(
-                            currentTileSetStrings.unitsLocation,
-                            attacker.getUnitType().name,
-                            "-attack-"
-                        )
-                if (ImageGetter.imageExists(unitSpecificAttackAnimationLocation+"1")) return unitSpecificAttackAnimationLocation
+                val unitSpecificAttackAnimationLocation = currentTileSetStrings.getLocation(attacker.getName())
+                if (ImageGetter.imageExists(unitSpecificAttackAnimationLocation + "1"))
+                    return unitSpecificAttackAnimationLocation
             }
 
-            val unitTypeAttackAnimationLocation =
-                    currentTileSetStrings.getString(currentTileSetStrings.unitsLocation, attacker.getUnitType().name, "-attack-")
-
-            if (ImageGetter.imageExists(unitTypeAttackAnimationLocation+"1")) return unitTypeAttackAnimationLocation
+            val unitTypeAttackAnimationLocation = currentTileSetStrings.getLocation(attacker.getUnitType().name)
+            if (ImageGetter.imageExists(unitTypeAttackAnimationLocation + "1"))
+                return unitTypeAttackAnimationLocation
             return null
         }
     }
 
+
+    /** The animation for the Damage labels */
+    private class DamageLabelAnimation(actor: WidgetGroup) : TemporalAction(damageLabelDuration) {
+        val startX = actor.x
+        val startY = actor.y
+
+        /* A tested version with smooth scale-out in addition to the alpha fade
+        val width = actor.width
+        val height = actor.height
+        init {
+            actor.isTransform = true
+        }
+        override fun update(percent: Float) {
+            actor.color.a = Interpolation.fade.apply(1f - percent)
+            val scale = Interpolation.smooth.apply(1f - percent)
+            actor.setScale(scale)
+            val x = startX + (1f - scale) * width / 2
+            val y = startY + (1f - scale) * height / 2 +
+                    Interpolation.smooth.apply(percent) * damageLabelDisplacement
+            actor.setPosition(x, y)
+        }
+        */
+
+        override fun update(percent: Float) {
+            actor.color.a = Interpolation.fade.apply(1f - percent)
+            actor.setPosition(startX, startY + percent * damageLabelDisplacement)
+        }
+        override fun end() {
+            actor.remove()
+        }
+    }
+
+
     fun WorldScreen.battleAnimation(
         attacker: ICombatant, damageToAttacker: Int,
         defender: ICombatant, damageToDefender: Int
@@ -97,8 +155,7 @@ object BattleTableHelpers {
                     if (combatant.isCity()) {
                         val icon = tileGroup.layerMisc.improvementIcon
                         if (icon != null) yield (icon)
-                    }
-                    else {
+                    } else if (!combatant.isAirUnit()) {
                         val slot = if (combatant.isCivilian()) 0 else 1
                         yieldAll((tileGroup.layerUnitArt.getChild(slot) as Group).children)
                     }
@@ -108,18 +165,28 @@ object BattleTableHelpers {
                 sequence {
                     if (damageToDefender != 0) yieldAll(getMapActorsForCombatant(defender))
                     if (damageToAttacker != 0) yieldAll(getMapActorsForCombatant(attacker))
-                }.mapTo(arrayListOf()) { it to it.color.cpy() }.toMap()
+                }.associateWith { it.color.cpy() }
 
         val actorsToMove = getMapActorsForCombatant(attacker).toList()
 
         val attackVectorHexCoords = defender.getTile().position.cpy().sub(attacker.getTile().position)
         val attackVectorWorldCoords = HexMath.hex2WorldCoords(attackVectorHexCoords)
             .nor()  // normalize vector to length of "1"
-            .scl(10f) // we want 10 pixel movement
+            .scl(moveActorsDisplacement)
+
+        val attackerGroup = mapHolder.tileGroups[attacker.getTile()]!!
+        val defenderGroup = mapHolder.tileGroups[defender.getTile()]!!
+        val hideDefenderDamage = defender.isDefeated() &&
+                attacker.getTile().position == defender.getTile().position
 
         stage.addAction(
             Actions.sequence(
                 MoveActorsAction(actorsToMove, attackVectorWorldCoords),
+                Actions.run {
+                    createDamageLabel(damageToAttacker, attackerGroup)
+                    if (!hideDefenderDamage)
+                        createDamageLabel(damageToDefender, defenderGroup)
+                },
                 Actions.parallel( // While the unit is moving back to its normal position, we flash the damages on both units
                     MoveActorsAction(actorsToMove, attackVectorWorldCoords.cpy().scl(-1f)),
                     AttackAnimationAction(attacker,
@@ -137,8 +204,27 @@ object BattleTableHelpers {
                     )
                 )
         ))
+    }
 
+    private fun createDamageLabel(damage: Int, target: Actor) {
+        if (damage == 0) return
 
+        val label = (-damage).toString().toLabel(Color.RED, damageLabelFontSize, Align.topLeft, true)
+        label.touchable = Touchable.disabled
+        val shadow = (-damage).toString().toLabel(Color.BLACK, damageLabelFontSize, Align.bottomRight, true)
+        shadow.touchable = Touchable.disabled
+
+        val container = Stack(shadow, label)
+        container.touchable = Touchable.disabled
+
+        container.pack()
+        // The +1f is what displaces the shadow under the red label
+        container.setSize(container.width + 1f, container.height + 1f)
+        val targetRight = target.run { localToStageCoordinates(Vector2(width, height * 0.5f)) }
+        container.setPosition(targetRight.x, targetRight.y, Align.center)
+        target.stage.addActor(container)
+
+        container.addAction(DamageLabelAnimation(container))
     }
 
     fun getHealthBar(maxHealth: Int, currentHealth: Int, maxRemainingHealth: Int, minRemainingHealth: Int): Table {
@@ -151,11 +237,11 @@ object BattleTableHelpers {
 
         val damagedHealth = ImageGetter.getDot(Color.FIREBRICK)
         if (UncivGame.Current.settings.continuousRendering) {
-            damagedHealth.addAction(Actions.repeat(
-                RepeatAction.FOREVER, Actions.sequence(
+            damagedHealth.addAction(Actions.forever(Actions.sequence(
                 Actions.color(Color.BLACK, 0.7f),
                 Actions.color(Color.FIREBRICK, 0.7f)
-            ))) }
+            )))
+        }
 
         val maybeDamagedHealth = ImageGetter.getDot(Color.ORANGE)
 
